#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Drawing;

namespace HeuristicLab.Optimization.Networks {
  [Item("Port", "Abstract base class for ports of an optimization network node.")]
  [StorableClass]
  public abstract class Port<T> : NamedItem, IPort<T> where T : class, IItem {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Field; }
    }

    [Storable]
    protected T value;
    public virtual T Value {
      get { return value; }
      protected set {
        DeregisterValueEvents();
        this.value = value;
        RegisterValueEvents();
        OnValueChanged();
      }
    }
    object IPort.Value {
      get { return Value; }
    }

    [StorableConstructor]
    protected Port(bool deserializing) : base(deserializing) { }
    protected Port(Port<T> original, Cloner cloner)
      : base(original, cloner) {
      value = cloner.Clone(original.value);
      RegisterValueEvents();
    }
    protected Port() : base("Port") { }
    protected Port(string name) : base(name) { }
    protected Port(string name, string description) : base(name, description) { }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterValueEvents();
    }

    public override string ToString() {
      return Name + ": " + (Value != null ? Value.ToString() : "null");
    }

    public event EventHandler ValueChanged;
    protected virtual void OnValueChanged() {
      var handler = ValueChanged;
      if (handler != null) handler(this, EventArgs.Empty);
      OnToStringChanged();
    }

    protected virtual void RegisterValueEvents() {
      if (value != null) {
        value.ToStringChanged += new EventHandler(Value_ToStringChanged);
      }
    }
    protected virtual void DeregisterValueEvents() {
      if (value != null) {
        value.ToStringChanged -= new EventHandler(Value_ToStringChanged);
      }
    }
    private void Value_ToStringChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }
  }
}
